@@ -679,7 +679,7 @@ static uint8_t quantizeCheck(uint8_t row)
 		{
 			if (song->tick > song->speed>>1)
 			{
-				row = (row + 1) & 0x3F;
+				row = (row + 1) & 63;
 				editor.didQuantize = true;
 			}
 		}
@@ -793,6 +793,7 @@ static void jamAndPlaceSample(SDL_Scancode scancode, bool normalMode)
 				gotoNextMulti();
 		}
 
+		// PT quirk: spectrum analyzer is still handled here even if channel is muted
 		updateSpectrumAnalyzer(s->volume, tempPeriod);
 	}
 	else if (noteVal == -2)
