@@ -42,7 +42,7 @@
 
 module_t *song = NULL; // globalized
 
-static bool backupMadeAfterCrash, didDropFile;
+static bool backupMadeAfterCrash;
 
 #ifdef _WIN32
 #define SYSMSG_FILE_ARG (WM_USER + 1)
@@ -286,6 +286,13 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	// setup GUI text pointers (for recently allocated song structure)
+	editor.currEditPatternDisp = &song->currPattern;
+	editor.currPosDisp = &song->currOrder;
+	editor.currPatternDisp = &song->header.order[0];
+	editor.currPosEdPattDisp = &song->header.order[0];
+	editor.currLengthDisp = &song->header.numOrders;
+
 	if (!initScopes())
 	{
 		cleanUp();
@@ -365,6 +372,7 @@ int main(int argc, char *argv[])
 
 static void handleInput(void)
 {
+	
 	SDL_Event event;
 	while (SDL_PollEvent(&event))
 	{
@@ -401,16 +409,26 @@ static void handleInput(void)
 		}
 		else if (event.type == SDL_DROPFILE)
 		{
-			// kludge: allow focus-clickthrough after drag-n-drop
-			SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "1");
-			didDropFile = true;
-
 			loadDroppedFile(event.drop.file, (uint32_t)strlen(event.drop.file), false, true);
 			SDL_free(event.drop.file);
+
+			SDL_RestoreWindow(video.window);
+			SDL_RaiseWindow(video.window);
 		}
-		if (event.type == SDL_QUIT)
+		else if (event.type == SDL_QUIT)
 		{
-			handleSigTerm();
+#ifdef __APPLE__
+			/* On Mac, command+Q sends a quit signal to the program.
+			** However, command+Q is also one of the transpose keys in this ProTracker port.
+			** Ignore the signal if command+Q was pressed.
+			*/
+			if (!editor.macCmdQIssued)
+#endif
+				handleSigTerm();
+
+#ifdef __APPLE__
+			editor.macCmdQIssued = false; // read note above
+#endif
 		}
 		else if (event.type == SDL_KEYUP)
 		{
@@ -432,13 +450,6 @@ static void handleInput(void)
 
 				ui.introTextShown = false;
 			}
-
-			// kludge: we drag-n-dropped a file before this mouse click release, restore focus-clickthrough mode
-			if (didDropFile)
-			{
-				didDropFile = false;
-				SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "0");
-			}
 		}
 		else if (event.type == SDL_MOUSEBUTTONDOWN)
 		{
@@ -475,9 +486,9 @@ static bool initializeVars(void)
 
 	editor.repeatKeyFlag = (SDL_GetModState() & KMOD_CAPS) ? true : false;
 
-	// set key repeat rate to 49.9204Hz (Amiga PAL vblank rate)
-	const double dVblankHzRatio = AMIGA_PAL_VBLANK_HZ / (double)VBLANK_HZ;
-	keyb.repeatDelta = (uint64_t)floor((UINT32_MAX+1.0) * dVblankHzRatio);
+	// 0.52 fixed-point delta for Amiga PAL vblank (~49.92Hz) at VBLANK_HZ (60.0Hz)
+	const double dRatio = AMIGA_PAL_VBLANK_HZ / (double)VBLANK_HZ;
+	video.amigaVblankDelta = (uint64_t)((dRatio * (1ULL << 52)) + 0.5);
 
 	strcpy(editor.mixText, "MIX 01+02 TO 03");
 
@@ -843,6 +854,12 @@ static void handleSysMsg(SDL_Event inputEvent)
 
 				UnmapViewOfFile(sharedMemBuf);
 				sharedMemBuf = NULL;
+
+				if (video.window != NULL)
+				{
+					SDL_RestoreWindow(video.window);
+					SDL_RaiseWindow(video.window);
+				}
 			}
 
 			CloseHandle(hMapFile);
