@@ -33,6 +33,7 @@
 #include "pt2_sampling.h"
 #include "pt2_chordmaker.h"
 #include "pt2_mod2wav.h"
+#include "pt2_audio.h"
 
 typedef struct sprite_t
 {
@@ -543,9 +544,7 @@ void updateSongInfo2(void) // two middle rows of screen, always present
 
 	// playback timer
 
-	const uint32_t ms1024 = editor.musicTime64 >> 32; // milliseconds (scaled from 1000 to 1024)
-
-	uint32_t seconds = ms1024 >> 10;
+	uint32_t seconds = editor.playbackSeconds;
 	if (seconds <= 5999) // below 100 minutes (99:59 is max for the UI)
 	{
 		const uint32_t MI_TimeM = seconds / 60;
@@ -1721,57 +1720,77 @@ void flipFrame(void)
 			hpc_Wait(&video.vblankHpc);
 #endif
 	}
+
+	editor.framesPassed++;
+
+	/* Reset audio/video sync timestamp every half an hour to prevent
+	** possible sync drifting after hours of playing a song without
+	** a single song stop (resets timestamp) in-between.
+	*/
+	if (editor.framesPassed >= VBLANK_HZ*60*30)
+		audio.resetSyncTickTimeFlag = true;
 }
 
-void updateSpectrumAnalyzer(int8_t vol, int16_t period)
+void updateSpectrumAnalyzer(uint8_t vol, uint16_t period)
 {
-	const uint8_t maxHeight = SPECTRUM_BAR_HEIGHT + 1; // +1 because of audio latency - allows full height to be seen
-
-	if (ui.visualizerMode != VISUAL_SPECTRUM || vol <= 0)
+	if (ui.visualizerMode != VISUAL_SPECTRUM || vol == 0)
 		return;
 
-	uint16_t scaledVol = ((uint16_t)vol * 24576) >> 16; // scaledVol = vol / 2.66667 (0..64 -> 0..24)
+	/* This routine is buggy in real PT. Behavior for periods outside of 108..907 is undefined.
+	** Behavior for sane input outside of 113..856 (B-3 finetune > 0 and C-1 finetune < 0)
+	** seems to be defined, and we simulate this.
+	*/
+	if (period < 108 || period > 907)
+		return; // we don't really know what to do here
 
-	period = CLAMP(period, 113, 856);
-	period -= 113;
+	// C-1 with finetune < 0 is ignored (confirmed behavior)
+	if (period > 856)
+		return;
 
-	uint32_t scaledNote = 743 - period;
-	scaledNote *= scaledNote;
-	scaledNote /= 25093; // scaledNote now ranges 0..22, no need to clamp
+	// B-3 with finetune > 0 behaves like finetune 0 (confirmed behavior)
+	if (period < 113)
+		period = 113;
 
-	// increment main spectrum bar
-	editor.spectrumVolumes[scaledNote] += (uint8_t)scaledVol;
-	if (editor.spectrumVolumes[scaledNote] > maxHeight)
-		editor.spectrumVolumes[scaledNote] = maxHeight;
+	// just in case
+	if (vol > 64)
+		vol = 64;
 
-	// increment left side of spectrum bar with half volume
-	if (scaledNote > 0)
+	const uint8_t vol24 = (vol * 24) >> 6; // 0..64 -> 0..24
+
+	// convert period from log (113..856) to linear (0..22)
+	period = (856 - 113) - (period - 113); // 0..743 (inverted)
+	const uint32_t index = (period * period) / 25093; // 0..22 (25093 = round[743^2 / 22])
+
+	// increment bar
+	editor.spectrumVolumes[index] += vol24;
+	if (editor.spectrumVolumes[index] > SPECTRUM_BAR_HEIGHT)
+		editor.spectrumVolumes[index] = SPECTRUM_BAR_HEIGHT;
+
+	// increment left and right neighbor bars with half the volume
+
+	if (index > 0)
 	{
-		editor.spectrumVolumes[scaledNote-1] += (uint8_t)(scaledVol >> 1);
-		if (editor.spectrumVolumes[scaledNote-1] > maxHeight)
-			editor.spectrumVolumes[scaledNote-1] = maxHeight;
+		editor.spectrumVolumes[index-1] += vol24 / 2;
+		if (editor.spectrumVolumes[index-1] > SPECTRUM_BAR_HEIGHT)
+			editor.spectrumVolumes[index-1] = SPECTRUM_BAR_HEIGHT;
 	}
 
-	// increment right side of spectrum bar with half volume
-	if (scaledNote < SPECTRUM_BAR_NUM-1)
+	if (index < SPECTRUM_BAR_NUM-1)
 	{
-		editor.spectrumVolumes[scaledNote+1] += (uint8_t)(scaledVol >> 1);
-		if (editor.spectrumVolumes[scaledNote+1] > maxHeight)
-			editor.spectrumVolumes[scaledNote+1] = maxHeight;
+		editor.spectrumVolumes[index+1] += vol24 / 2;
+		if (editor.spectrumVolumes[index+1] > SPECTRUM_BAR_HEIGHT)
+			editor.spectrumVolumes[index+1] = SPECTRUM_BAR_HEIGHT;
 	}
 }
 
-void sinkVisualizerBars(void)
+void sinkVisualizerBars(void) // sinks visualizer bars @ 49.92Hz (Amiga PAL) rate
 {
-	// sink visualizer bars @ 49.92Hz (Amiga PAL) rate
+	static uint64_t counter50Hz; // pre-initialized to zero because of static
 
-	static uint64_t counter50Hz;
-	const uint64_t counter50HzDelta = (uint64_t)(((UINT32_MAX+1.0) * (AMIGA_PAL_VBLANK_HZ / (double)VBLANK_HZ)) + 0.5);
-
-	counter50Hz += counter50HzDelta; // 32.32 fixed-point counter
-	if (counter50Hz > UINT32_MAX)
+	counter50Hz += video.amigaVblankDelta; // 0.52 fixed-point
+	if (counter50Hz > 1ULL<<52)
 	{
-		counter50Hz &= UINT32_MAX;
+		counter50Hz &= (1ULL<<52)-1;
 
 		// sink VU-meters
 		for (int32_t i = 0; i < PAULA_VOICES; i++)
@@ -1925,6 +1944,38 @@ void toggleFullscreen(void)
 
 bool setupVideo(void)
 {
+	if (config.autoFitVideoScale)
+	{
+		int8_t i;
+		SDL_DisplayMode dm;
+
+		int32_t di = SDL_GetWindowDisplayIndex(video.window);
+		if (di < 0)
+			di = 0; // return display index 0 (default) on error
+
+		// find out which upscaling factor is the biggest to fit on screen
+		if (SDL_GetDesktopDisplayMode(di, &dm) == 0)
+		{
+			for (i = MAX_UPSCALE_FACTOR; i >= 1; i--)
+			{
+				// height test is slightly taller because of window title, window borders and taskbar/menu/dock
+				if (dm.w >= SCREEN_W*i && dm.h >= (SCREEN_H+64)*i)
+				{
+					config.videoScaleFactor = i;
+					break;
+				}
+			}
+
+			if (i == 0)
+				config.videoScaleFactor = 1; // 1x is not going to fit, but use 1x anyways...
+		}
+		else
+		{
+			// couldn't get screen resolution, set to 1x
+			config.videoScaleFactor = 1;
+		}
+	}
+
 	int32_t screenW = SCREEN_W * config.videoScaleFactor;
 	int32_t screenH = SCREEN_H * config.videoScaleFactor;
 
